@@ -22,7 +22,8 @@ import inspect
 import traceback
 import textwrap
 import os
-import ipc
+
+from . import ipc
 
 
 class CommandError(Exception):
@@ -80,7 +81,7 @@ class _Server(ipc.Server):
         selectors, name, args, kwargs = data
         try:
             obj = self.qtile.select(selectors)
-        except _SelectError, v:
+        except _SelectError as v:
             e = formatSelector([(v.name, v.sel)])
             s = formatSelector(selectors)
             return (ERROR, "No object %s in path '%s'" % (e, s))
@@ -90,14 +91,14 @@ class _Server(ipc.Server):
         self.qtile.log.info("Command: %s(%s, %s)" % (name, args, kwargs))
         try:
             return (SUCCESS, cmd(*args, **kwargs))
-        except CommandError, v:
+        except CommandError as v:
             return (ERROR, v.args[0])
-        except Exception, v:
+        except Exception as v:
             return (EXCEPTION, traceback.format_exc())
         self.qtile.conn.flush()
 
 
-class _Command:
+class _Command(object):
     def __init__(self, call, selectors, name):
         """
             :command A string command name specification
@@ -262,7 +263,7 @@ class CommandRoot(_CommandRoot):
             raise CommandException(val)
 
 
-class _Call:
+class _Call(object):
     def __init__(self, selectors, name, *args, **kwargs):
         """
             :command A string command name specification
@@ -423,16 +424,16 @@ class CommandObject(object):
             try:
                 return (True, str(eval(code)))
             except SyntaxError:
-                exec code
+                exec(code)
                 return (True, None)
         except:
             error = traceback.format_exc().strip().split("\n")[-1]
             return (False, error)
 
-    def cmd_function(self, function):
+    def cmd_function(self, function, *args, **kwargs):
         """Call a function with current object as argument"""
         try:
-            function(self)
+            function(self, *args, **kwargs)
         except Exception:
             error = traceback.format_exc()
             self.log.error('Exception calling "%s":\n%s' % (function, error))
