@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # coding: utf-8
 #
 # Copyright (c) 2008, Aldo Cortesi <aldo@corte.si>
@@ -26,7 +25,6 @@
 
 import os
 import sys
-import utils
 import traceback
 
 
@@ -42,16 +40,18 @@ class File(object):
                 # if variable wasn't set
                 config_directory = os.path.expanduser("~/.config")
             fname = os.path.join(config_directory, "qtile", "config.py")
-        elif fname == "default":
-            fname = utils.data.path("resources/default_config.py")
 
-        self.fname = fname
+        # We delay importing here to avoid a circular import issue when
+        # testing.
+        from .resources import default_config
 
-        if os.path.isfile(fname):
+        if fname == "default":
+            config = default_config
+        elif os.path.isfile(fname):
             try:
-                sys.path.insert(0, os.path.dirname(self.fname))
-                config = __import__(os.path.basename(self.fname)[:-3])
-            except Exception, v:
+                sys.path.insert(0, os.path.dirname(fname))
+                config = __import__(os.path.basename(fname)[:-3])
+            except Exception as v:
                 # On restart, user potentially has some windows open, but they
                 # screwed up their config. So as not to lose their apps, we
                 # just load the default config here.
@@ -84,9 +84,6 @@ class File(object):
             "wmname",
         ]
 
-        # We delay importing here to avoid a circular import issue when
-        # testing.
-        from resources import default_config
         for option in config_options:
             if hasattr(config, option):
                 v = getattr(config, option)
