@@ -1,4 +1,31 @@
-import gobject
+# Copyright (c) 2011-2012 Florian Mounier
+# Copyright (c) 2012-2014 roger
+# Copyright (c) 2012 Craig Barnes
+# Copyright (c) 2012-2014 Tycho Andersen
+# Copyright (c) 2013 Tao Sauvage
+# Copyright (c) 2014 ramnes
+# Copyright (c) 2014 Sebastian Kricner
+# Copyright (c) 2014 Sean Vig
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
+import collections
 
 import libqtile.hook
 from libqtile.config import Key
@@ -22,7 +49,7 @@ def simple_key_binder(mod, keynames=None):
             keys = keynames
         else:
             # keys 1 to 9 and 0
-            keys = map(str, range(1, 10) + [0])
+            keys = list(map(str, list(range(1, 10)) + [0]))
 
         # bind all keys
         for keyname, group in zip(keys, dgroup.qtile.groups):
@@ -71,18 +98,22 @@ class DGroups(object):
         self.timeout = {}
 
     def add_rule(self, rule, last=True):
-        self.rules_map[self.last_rule_id] = rule
+        rule_id = self.last_rule_id
+        self.rules_map[rule_id] = rule
         if last:
             self.rules.append(rule)
         else:
             self.rules.insert(0, rule)
         self.last_rule_id += 1
-        return self.last_rule_id
+        return rule_id
 
-    def remove_rule(self, rule_id=None):
-        rule = self.rules[rule_id]
-        self.rules.remove(rule)
-        del self.rules[rule_id]
+    def remove_rule(self, rule_id):
+        rule = self.rules_map.get(rule_id, None)
+        if rule:
+            self.rules.remove(rule)
+            del self.rules_map[rule_id]
+        else:
+            self.qtile.log.warn('Rule "%s" not found' % rule_id)
 
     def add_dgroup(self, group, start=False):
         self.groupMap[group.name] = group
@@ -118,13 +149,17 @@ class DGroups(object):
     def _add(self, client):
         if client in self.timeout:
             self.qtile.log.info('Remove dgroup source')
-            gobject.source_remove(self.timeout[client])
-            del(self.timeout[client])
+            self.timeout.pop(client).cancel()
 
         # ignore static windows
         if client.defunct:
             return
 
+        # ignore windows whose groups is already set (e.g. from another hook or
+        # when it was set on state restore)
+        if client.group is not None:
+            return
+
         group_set = False
         intrusive = False
 
@@ -148,8 +183,8 @@ class DGroups(object):
                     group_obj = self.qtile.groupMap[rule.group]
                     group = self.groupMap.get(rule.group)
                     if group and group_added:
-                        for k, v in group.layout_opts.iteritems():
-                            if callable(v):
+                        for k, v in list(group.layout_opts.items()):
+                            if isinstance(v, collections.Callable):
                                 v(group_obj.layout)
                             else:
                                 setattr(group_obj.layout, k, v)
@@ -206,7 +241,6 @@ class DGroups(object):
 
         # Wait the delay until really delete the group
         self.qtile.log.info('Add dgroup timer')
-        self.timeout[client] = gobject.timeout_add_seconds(
-            self.delay,
-            delete_client
+        self.timeout[client] = self.qtile._eventloop.call_later(
+            self.delay, delete_client
         )
