@@ -1,10 +1,39 @@
-import command
-import hook
-import window
-import utils
+# Copyright (c) 2012-2014 Tycho Andersen
+# Copyright (c) 2013 xarvh
+# Copyright (c) 2013 roger
+# Copyright (c) 2013 Tao Sauvage
+# Copyright (c) 2014 ramnes
+# Copyright (c) 2014 Sean Vig
+# Copyright (c) 2014 dequis
+# Copyright (c) 2015 Dario Giovannetti
+# Copyright (c) 2015 Alexander Lozovskoy
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 import contextlib
-import xcb
-import xcb.xproto
+import xcffib
+import xcffib.xproto
+
+from . import command
+from . import hook
+from . import window
+from . import utils
 
 
 class _Group(command.CommandObject):
@@ -20,14 +49,19 @@ class _Group(command.CommandObject):
         self.qtile = None
         self.layouts = []
         self.floating_layout = None
-        self.currentWindow = None
+        # self.focusHistory lists the group's windows in the order they
+        # received focus, from the oldest (first item) to the currently
+        # focused window (last item); NB the list does *not* contain any
+        # windows that never received focus; refer to self.windows for the
+        # complete set
+        self.focusHistory = []
         self.screen = None
         self.currentLayout = None
 
     def _configure(self, layouts, floating_layout, qtile):
         self.screen = None
         self.currentLayout = 0
-        self.currentWindow = None
+        self.focusHistory = []
         self.windows = set()
         self.qtile = qtile
         self.layouts = [i.clone(self) for i in layouts]
@@ -36,6 +70,34 @@ class _Group(command.CommandObject):
             self.layout = self.customLayout
             self.customLayout = None
 
+    @property
+    def currentWindow(self):
+        try:
+            return self.focusHistory[-1]
+        except IndexError:
+            # no window has focus
+            return None
+
+    @currentWindow.setter
+    def currentWindow(self, win):
+        try:
+            self.focusHistory.remove(win)
+        except ValueError:
+            # win has never received focus before
+            pass
+        self.focusHistory.append(win)
+
+    def _remove_from_focus_history(self, win):
+        try:
+            index = self.focusHistory.index(win)
+        except ValueError:
+            # win has never received focus
+            return False
+        else:
+            del self.focusHistory[index]
+            # return True if win was the last item (i.e. it was currentWindow)
+            return index == len(self.focusHistory)
+
     @property
     def layout(self):
         return self.layouts[self.currentLayout]
@@ -81,7 +143,7 @@ class _Group(command.CommandObject):
         moving warp to it.
         """
         if self.screen and len(self.windows):
-            with self.disableMask(xcb.xproto.EventMask.EnterWindow):
+            with self.disableMask(xcffib.xproto.EventMask.EnterWindow):
                 normal = [x for x in self.windows if not x.floating]
                 floating = [
                     x for x in self.windows
@@ -119,9 +181,9 @@ class _Group(command.CommandObject):
 
     def hide(self):
         self.screen = None
-        with self.disableMask(xcb.xproto.EventMask.EnterWindow |
-                              xcb.xproto.EventMask.FocusChange |
-                              xcb.xproto.EventMask.LeaveWindow):
+        with self.disableMask(xcffib.xproto.EventMask.EnterWindow |
+                              xcffib.xproto.EventMask.FocusChange |
+                              xcffib.xproto.EventMask.LeaveWindow):
             for i in self.windows:
                 i.hide()
             self.layout.hide()
@@ -148,21 +210,18 @@ class _Group(command.CommandObject):
         if win:
             if win not in self.windows:
                 return
+            self.currentWindow = win
+            if win.floating:
+                for l in self.layouts:
+                    l.blur()
+                self.floating_layout.focus(win)
             else:
-                self.currentWindow = win
-                if win.floating:
-                    for l in self.layouts:
-                        l.blur()
-                    self.floating_layout.focus(win)
-                else:
-                    self.floating_layout.blur()
-                    for l in self.layouts:
-                        l.focus(win)
-        else:
-            self.currentWindow = None
-        hook.fire("focus_change")
-        # !!! note that warp isn't hooked up now
-        self.layoutAll(warp)
+                self.floating_layout.blur()
+                for l in self.layouts:
+                    l.focus(win)
+            hook.fire("focus_change")
+            # !!! note that warp isn't hooked up now
+            self.layoutAll(warp)
 
     def info(self):
         return dict(
@@ -175,12 +234,12 @@ class _Group(command.CommandObject):
             screen=self.screen.index if self.screen else None
         )
 
-    def add(self, win):
+    def add(self, win, focus=True):
         hook.fire("group_window_add")
         self.windows.add(win)
         win.group = self
         try:
-            if win.window.get_net_wm_state() == 'fullscreen' and \
+            if 'fullscreen' in win.window.get_net_wm_state() and \
                     self.qtile.config.auto_fullscreen:
                 win._float_state = window.FULLSCREEN
             elif self.floating_layout.match(win):
@@ -188,50 +247,58 @@ class _Group(command.CommandObject):
                 # because it's too early
                 # so just set the flag underneath
                 win._float_state = window.FLOATING
-        except (xcb.xproto.BadWindow, xcb.xproto.BadAccess):
+        except (xcffib.xproto.WindowError, xcffib.xproto.AccessError):
             pass  # doesn't matter
         if win.floating:
             self.floating_layout.add(win)
         else:
             for i in self.layouts:
                 i.add(win)
-        self.focus(win, True)
+        if focus:
+            self.focus(win, True)
 
     def remove(self, win):
         self.windows.remove(win)
+        hadfocus = self._remove_from_focus_history(win)
         win.group = None
         nextfocus = None
         if win.floating:
             nextfocus = self.floating_layout.remove(win)
-            if nextfocus is None:
-                nextfocus = self.layout.focus_first()
-            if nextfocus is None:
-                nextfocus = self.floating_layout.focus_first()
+            if not hadfocus:
+                # For example a notification
+                return
+            nextfocus = nextfocus or \
+                self.currentWindow or \
+                self.layout.focus_first() or \
+                self.floating_layout.focus_first()
         else:
             for i in self.layouts:
                 if i is self.layout:
                     nextfocus = i.remove(win)
                 else:
                     i.remove(win)
-            if nextfocus is None:
-                nextfocus = self.floating_layout.focus_first()
-            if nextfocus is None:
-                nextfocus = self.layout.focus_first()
+            if not hadfocus:
+                return
+            nextfocus = nextfocus or \
+                self.floating_layout.focus_first() or \
+                self.currentWindow or \
+                self.layout.focus_first()
         self.focus(nextfocus, True)
         # else: TODO: change focus
 
     def mark_floating(self, win, floating):
-        if floating and win in self.floating_layout.clients:
-            # already floating
-            pass
-        elif floating:
-            for i in self.layouts:
-                i.remove(win)
+        if floating:
+            if win in self.floating_layout.clients:
+                # already floating
+                pass
+            else:
+                for i in self.layouts:
+                    i.remove(win)
+                    if win is self.currentWindow:
+                        i.blur()
+                self.floating_layout.add(win)
                 if win is self.currentWindow:
-                    i.blur()
-            self.floating_layout.add(win)
-            if win is self.currentWindow:
-                self.floating_layout.focus(win)
+                    self.floating_layout.focus(win)
         else:
             self.floating_layout.remove(win)
             self.floating_layout.blur()
@@ -243,7 +310,7 @@ class _Group(command.CommandObject):
 
     def _items(self, name):
         if name == "layout":
-            return (True, range(len(self.layouts)))
+            return (True, list(range(len(self.layouts))))
         elif name == "window":
             return (True, [i.window.wid for i in self.windows])
         elif name == "screen":
@@ -321,12 +388,12 @@ class _Group(command.CommandObject):
     def nextGroup(self, skip_empty=False, skip_managed=False):
         return self._dirGroup(1, skip_empty, skip_managed)
 
-    def cmd_unminimise_all(self):
+    def cmd_unminimize_all(self):
         """
             Unminimise all windows in this group.
         """
         for w in self.windows:
-            w.minimised = False
+            w.minimized = False
         self.layoutAll()
 
     def cmd_next_window(self):
