@@ -17,7 +17,7 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-from base import SingleWindow
+from .base import SingleWindow
 
 
 class Max(SingleWindow):
@@ -33,14 +33,14 @@ class Max(SingleWindow):
         SingleWindow.__init__(self, **config)
         self.clients = []
         self.add_defaults(Max.defaults)
-        self.focused = None
+        self.current = None
 
     def _get_window(self):
-        return self.focused
+        return self.current
 
     def focus(self, client):
         self.group.layoutAll()
-        self.focused = client
+        self.current = client
 
     def focus_first(self):
         if self.clients:
@@ -53,6 +53,8 @@ class Max(SingleWindow):
     def focus_next(self, window):
         if not self.clients:
             return
+        if window is None:
+            return
         if window != self._get_window():
             self.focus(window)
         idx = self.clients.index(window)
@@ -62,6 +64,8 @@ class Max(SingleWindow):
     def focus_previous(self, window):
         if not self.clients:
             return
+        if window is None:
+            return
         if window != self._get_window():
             self.focus(window)
         idx = self.clients.index(window)
@@ -69,11 +73,11 @@ class Max(SingleWindow):
             return self.clients[idx - 1]
 
     def up(self):
-        client = self.focus_previous(self.focused) or self.focus_last()
+        client = self.focus_previous(self.current) or self.focus_last()
         self.group.focus(client, False)
 
     def down(self):
-        client = self.focus_next(self.focused) or self.focus_first()
+        client = self.focus_next(self.current) or self.focus_first()
         self.group.focus(client, False)
 
     def clone(self, group):
@@ -82,17 +86,33 @@ class Max(SingleWindow):
         return c
 
     def add(self, client):
-        self.clients.insert(0, client)
+        try:
+            idx = self.clients.index(self._get_window())
+        except ValueError:
+            self.clients.append(client)
+        else:
+            self.clients.insert(idx + 1, client)
 
     def remove(self, client):
-        if client not in self.clients:
+        try:
+            idx = self.clients.index(client)
+        except ValueError:
             return
-        self.clients.remove(client)
-        if self.clients:
-            return self.clients[0]
+        else:
+            del self.clients[idx]
+
+        if client is self.current:
+            try:
+                # The previous client must become current
+                # if idx == 0, using self.clients[-1] is indeed correct
+                self.current = self.clients[idx - 1]
+            except IndexError:
+                self.current = None
+
+        return self.current
 
     def configure(self, client, screen):
-        if self.clients and client is self.focused:
+        if self.clients and client is self.current:
             client.place(
                 screen.x,
                 screen.y,
