@@ -17,13 +17,18 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-from base import Layout
+
+from __future__ import division
+
+from .base import Layout
 from .. import utils
 
 
 class _WinStack(object):
-    split = False
-    _current = 0
+    def __init__(self, autosplit=False):
+        self.split = autosplit
+        self._current = 0
+        self.lst = []
 
     @property
     def current(self):
@@ -42,9 +47,6 @@ class _WinStack(object):
             return None
         return self.lst[self.current]
 
-    def __init__(self):
-        self.lst = []
-
     def toggleSplit(self):
         self.split = False if self.split else True
 
@@ -88,7 +90,7 @@ class _WinStack(object):
         if client not in self.lst:
             return
         idx = self.lst.index(client)
-        self.lst.remove(client)
+        del self.lst[idx]
         if idx > self.current:
             self.current -= 1
         else:
@@ -108,8 +110,8 @@ class _WinStack(object):
     def __contains__(self, client):
         return client in self.lst
 
-    def __repr__(self):
-        return "_WinStack(%s, %s)" % (
+    def __str__(self):
+        return "_WinStack: %s, %s" % (
             self.current, str([i.name for i in self])
         )
 
@@ -143,10 +145,8 @@ class Stack(Layout):
     def __init__(self, **config):
         Layout.__init__(self, **config)
         self.add_defaults(Stack.defaults)
-        self.stacks = [_WinStack() for i in range(self.num_stacks)]
-        for stack in self.stacks:
-            if self.autosplit:
-                stack.split = True
+        self.stacks = [_WinStack(autosplit=self.autosplit)
+                    for i in range(self.num_stacks)]
 
     @property
     def currentStack(self):
@@ -169,10 +169,7 @@ class Stack(Layout):
     def clone(self, group):
         c = Layout.clone(self, group)
         # These are mutable
-        c.stacks = [_WinStack() for i in self.stacks]
-        for stack in c.stacks:
-            if self.autosplit:
-                stack.split = True
+        c.stacks = [_WinStack(autosplit=self.autosplit) for i in self.stacks]
         return c
 
     def _findNext(self, lst, offset):
@@ -296,12 +293,12 @@ class Stack(Layout):
         else:
             px = self.group.qtile.colorPixel(self.border_normal)
 
-        columnWidth = int(screen.width / float(len(self.stacks)))
+        columnWidth = int(screen.width / len(self.stacks))
         xoffset = screen.x + i * columnWidth
         winWidth = columnWidth - 2 * self.border_width
 
         if s.split:
-            columnHeight = int(screen.height / float(len(s)))
+            columnHeight = int(screen.height / len(s))
             winHeight = columnHeight - 2 * self.border_width
             yoffset = screen.y + s.index(client) * columnHeight
             client.place(
@@ -383,7 +380,7 @@ class Stack(Layout):
         """
             Add another stack to the layout.
         """
-        newstack = _WinStack()
+        newstack = _WinStack(autosplit=self.autosplit)
         if self.autosplit:
             newstack.split = True
         self.stacks.append(newstack)
