@@ -1,14 +1,47 @@
+# Copyright (c) 2008, 2010 Aldo Cortesi
+# Copyright (c) 2009 Ben Duffield
+# Copyright (c) 2010 aldo
+# Copyright (c) 2010-2012 roger
+# Copyright (c) 2011 Florian Mounier
+# Copyright (c) 2011 Kenji_Takahashi
+# Copyright (c) 2011-2015 Tycho Andersen
+# Copyright (c) 2012-2013 dequis
+# Copyright (c) 2012 Craig Barnes
+# Copyright (c) 2013 xarvh
+# Copyright (c) 2013 Tao Sauvage
+# Copyright (c) 2014 Sean Vig
+# Copyright (c) 2014 Filipe Nepomuceno
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 from .. import bar, hook
-import base
+from . import base
 
 
 class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
     defaults = [
         ("borderwidth", 3, "Current group border width"),
+        ("center_aligned", False, "center-aligned group box"),
     ]
 
     def __init__(self, **config):
-        base._TextBox.__init__(self, bar.CALCULATED, **config)
+        base._TextBox.__init__(self, width=bar.CALCULATED, **config)
         self.add_defaults(_GroupBase.defaults)
         self.add_defaults(base.PaddingMixin.defaults)
         self.add_defaults(base.MarginMixin.defaults)
@@ -48,6 +81,7 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
         hook.subscribe.setgroup(hook_response)
         hook.subscribe.group_window_add(hook_response)
         hook.subscribe.current_screen_change(hook_response)
+        hook.subscribe.changegroup(hook_response)
 
     def drawbox(self, offset, text, bordercolor, textcolor, rounded=False,
                 block=False, width=None):
@@ -63,16 +97,23 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
             self.padding_x,
             self.padding_y
         )
+        y = self.margin_y
+        if self.center_aligned:
+            for t in base.MarginMixin.defaults:
+                if t[0] == 'margin':
+                    y += (self.bar.height - framed.height) / 2 - t[1]
+                    break
         if block:
-            framed.draw_fill(offset, self.margin_y, rounded)
+            framed.draw_fill(offset, y, rounded)
         else:
-            framed.draw(offset, self.margin_y, rounded)
+            framed.draw(offset, y, rounded)
 
 
 class AGroupBox(_GroupBase):
     """
         A widget that graphically displays the current group.
     """
+    orientations = base.ORIENTATION_HORIZONTAL
     defaults = [("border", "000000", "group box border color")]
 
     def __init__(self, **config):
@@ -82,23 +123,24 @@ class AGroupBox(_GroupBase):
     def button_press(self, x, y, button):
         self.bar.screen.cmd_nextgroup()
 
-    def calculate_width(self):
+    def calculate_length(self):
         return self.box_width(self.qtile.groups)
 
     def draw(self):
         self.drawer.clear(self.background or self.bar.background)
-        e = (
+        e = next(
             i for i in self.qtile.groups
             if i.name == self.bar.screen.group.name
-        ).next()
+        )
         self.drawbox(self.margin_x, e.name, self.border, self.foreground)
-        self.drawer.draw(self.offset, self.width)
+        self.drawer.draw(offsetx=self.offset, width=self.width)
 
 
 class GroupBox(_GroupBase):
     """
         A widget that graphically displays the current group.
     """
+    orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
         ("active", "FFFFFF", "Active group font colour"),
         ("inactive", "404040", "Inactive group font colour"),
@@ -106,8 +148,8 @@ class GroupBox(_GroupBase):
         (
             "highlight_method",
             "border",
-            "Method of highlighting (one of 'border' or 'block') "
-            "Uses *_border color settings"
+            "Method of highlighting (one of 'border', 'block' or 'text') "
+            "Uses \*_border color settings"
         ),
         ("rounded", True, "To round or not to round borders"),
         (
@@ -181,7 +223,7 @@ class GroupBox(_GroupBase):
                 group.cmd_switch_groups(self.clicked.name)
                 self.clicked = None
 
-    def calculate_width(self):
+    def calculate_length(self):
         width = 0
         for g in self.qtile.groups:
             width += self.box_width([g])
@@ -198,14 +240,26 @@ class GroupBox(_GroupBase):
             is_block = (self.highlight_method == 'block')
 
             bw = self.box_width([g])
+
+            if self.group_has_urgent(g) and self.urgent_alert_method == "text":
+                text_color = self.urgent_text
+            elif g.windows:
+                text_color = self.active
+            else:
+                text_color = self.inactive
+
             if g.screen:
-                if self.bar.screen.group.name == g.name:
-                    if self.qtile.currentScreen == self.bar.screen:
-                        border = self.this_current_screen_border
-                    else:
-                        border = self.this_screen_border
+                if self.highlight_method == 'text':
+                    border = self.bar.background
+                    text_color = self.this_current_screen_border
                 else:
-                    border = self.other_screen_border
+                    if self.bar.screen.group.name == g.name:
+                        if self.qtile.currentScreen == self.bar.screen:
+                            border = self.this_current_screen_border
+                        else:
+                            border = self.this_screen_border
+                    else:
+                        border = self.other_screen_border
             elif self.group_has_urgent(g) and \
                     self.urgent_alert_method in ('border', 'block'):
                 border = self.urgent_border
@@ -214,21 +268,14 @@ class GroupBox(_GroupBase):
             else:
                 border = self.background or self.bar.background
 
-            if self.group_has_urgent(g) and self.urgent_alert_method == "text":
-                text = self.urgent_text
-            elif g.windows:
-                text = self.active
-            else:
-                text = self.inactive
-
             self.drawbox(
                 self.margin_x + offset,
                 g.name,
                 border,
-                text,
+                text_color,
                 self.rounded,
                 is_block,
                 bw - self.margin_x * 2 - self.padding_x * 2
             )
             offset += bw
-        self.drawer.draw(self.offset, self.width)
+        self.drawer.draw(offsetx=self.offset, width=self.width)
