@@ -1,14 +1,39 @@
+# Copyright (c) 2013 Jacob Mourelos
+# Copyright (c) 2014 Shepilov Vladislav
+# Copyright (c) 2014-2015 Sean Vig
+# Copyright (c) 2014 Tycho Andersen
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
+import re
 import subprocess
 from subprocess import CalledProcessError
-import base
-import re
+
+from . import base
 
 
 class KeyboardLayout(base.InLoopPollText):
     """
         Widget for changing and displaying the current keyboard layout.
-        It requires setxkbmap to be available in the sytem.
+        It requires setxkbmap to be available in the system.
     """
+    orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
         ("update_interval", 1, "Update time in seconds."),
         ("configured_keyboards", "us", "A list of predefined keyboard layouts "
@@ -32,7 +57,7 @@ class KeyboardLayout(base.InLoopPollText):
             new layout the first one in the list.
         """
 
-        current_keyboard = self.poll()
+        current_keyboard = self._get_keyboard()
         if current_keyboard in self.configured_keyboards:
             # iterate the list circularly
             next_keyboard = self.configured_keyboards[
@@ -43,20 +68,23 @@ class KeyboardLayout(base.InLoopPollText):
         self._set_keyboard(next_keyboard)
 
     def poll(self):
+        return self._get_keyboard().upper()
+
+    def _get_keyboard(self):
         """
             Return the currently used keyboard layout as a string.
             Examples: "us", "us dvorak".
             In case of error returns "unknown".
         """
         try:
-            xset_output = subprocess.check_output(["xset", "-q"])
-            keyboard = _Keyboard(self.configured_keyboards).get_keyboard_layout(xset_output).upper()
+            xset_output = self.call_process(["xset", "-q"])
+            keyboard = _Keyboard(self.configured_keyboards).get_keyboard_layout(xset_output)
             return str(keyboard)
         except CalledProcessError as e:
-            self.log.error('Can not change the keyboard layout: {0}'
+            self.log.error('Can not get the keyboard layout: {0}'
                            .format(e))
         except OSError as e:
-            self.log.error('Please, check that setxkbmap is available: {0}'
+            self.log.error('Please, check that xset is available: {0}'
                            .format(e))
         return "unknown"
 
