@@ -1,9 +1,42 @@
-import cairo
+# Copyright (c) 2012-2014 roger
+# Copyright (c) 2012-2015 Tycho Andersen
+# Copyright (c) 2013 dequis
+# Copyright (c) 2013 Tao Sauvage
+# Copyright (c) 2013 Craig Barnes
+# Copyright (c) 2014 Sean Vig
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
+from __future__ import division
+
+import cairocffi
 from .. import bar, hook
-import base
+from . import base
 
 
 class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
+    """
+        Displays the icon and name of each window in the current group.
+        Contrary to WindowTabs this is an interactive widget.
+        The window that currently has focus is highlighted.
+    """
+    orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
         ("font", "Arial", "Default font"),
         ("fontsize", None, "Font size. Calculated if None."),
@@ -20,7 +53,7 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             "highlight_method",
             "border",
             "Method of highlighting (one of 'border' or 'block') "
-            "Uses *_border color settings"
+            "Uses \*_border color settings"
         ),
         ("urgent_border", "FF0000", "Urgent border color"),
         (
@@ -101,9 +134,12 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             self.layout.width = width
 
     def drawbox(self, offset, text, bordercolor, textcolor, rounded=False,
-                block=False, width=None):
+                block=False, width=None, icon=None):
         self.drawtext(text, textcolor, width)
-        padding_x = [self.padding_x + self.icon_size + 4, self.padding_x]
+
+        icon_padding = (self.icon_size + 4) if icon else 0
+        padding_x = [self.padding_x + icon_padding, self.padding_x]
+
         framed = self.layout.framed(
             self.borderwidth,
             bordercolor,
@@ -115,6 +151,9 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
         else:
             framed.draw(offset, self.margin_y, rounded)
 
+        if icon:
+            self.draw_icon(icon, offset)
+
     def get_clicked(self, x, y):
         window = None
         new_width = width = 0
@@ -140,30 +179,33 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
                 window.cmd_bring_to_front()
 
     def get_window_icon(self, window):
+        if not window.icons:
+            return None
+
         cache = self._icons_cache.get(window.window.wid)
         if cache:
             return cache
 
         icons = sorted(
-            window.icons.iteritems(),
+            iter(window.icons.items()),
             key=lambda x: abs(self.icon_size - int(x[0].split("x")[0]))
         )
         icon = icons[0]
         width, height = map(int, icon[0].split("x"))
 
-        img = cairo.ImageSurface.create_for_data(
+        img = cairocffi.ImageSurface.create_for_data(
             icon[1],
-            cairo.FORMAT_ARGB32,
+            cairocffi.FORMAT_ARGB32,
             width,
             height
         )
 
-        surface = cairo.SurfacePattern(img)
+        surface = cairocffi.SurfacePattern(img)
 
-        scaler = cairo.Matrix()
+        scaler = cairocffi.Matrix()
 
         if height != self.icon_size:
-            sp = height / float(self.icon_size)
+            sp = height / self.icon_size
             height = self.icon_size
             width = width / sp
             scaler.scale(sp, sp)
@@ -171,15 +213,13 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
         self._icons_cache[window.window.wid] = surface
         return surface
 
-    def draw_icon(self, window, offset):
-        if not window.icons:
+    def draw_icon(self, surface, offset):
+        if not surface:
             return
 
         x = offset + self.padding_x + self.borderwidth + 2 + self.margin_x
         y = self.padding_y + self.borderwidth
 
-        surface = self.get_window_icon(window)
-
         self.drawer.ctx.save()
         self.drawer.ctx.translate(x, y)
         self.drawer.ctx.set_source(surface)
@@ -217,9 +257,9 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
                 self.foreground,
                 self.rounded,
                 self.highlight_method == 'block',
-                bw - self.margin_x * 2 - self.padding_x * 2
+                bw - self.margin_x * 2 - self.padding_x * 2,
+                icon=self.get_window_icon(w),
             )
-            self.draw_icon(w, offset)
 
             offset += bw + self.icon_size
-        self.drawer.draw(self.offset, self.width)
+        self.drawer.draw(offsetx=self.offset, width=self.width)
