@@ -18,15 +18,18 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import division
+
 import array
-import struct
 import contextlib
-import xcb.xcb
-from xcb.xproto import EventMask, StackMode, SetMode
-import xcb.xproto
-import command
-import utils
-import hook
+import inspect
+import traceback
+from xcffib.xproto import EventMask, StackMode, SetMode
+import xcffib.xproto
+
+from . import command
+from . import utils
+from . import hook
 
 
 # ICCM Constants
@@ -126,7 +129,7 @@ class _Window(command.CommandObject):
                 'w': g.width,
                 'h': g.height,
             }
-        except xcb.xproto.BadDrawable:
+        except xcffib.xproto.DrawableError:
             # Whoops, we were too early, so let's ignore it for now and get the
             # values on demand.
             self._x = None
@@ -141,6 +144,7 @@ class _Window(command.CommandObject):
         self.state = NormalState
         self.window_type = "normal"
         self._float_state = NOT_FLOATING
+        self._demands_attention = False
 
         self.hints = {
             'input': True,
@@ -163,9 +167,9 @@ class _Window(command.CommandObject):
         def get_attr(self):
             if getattr(self, "_" + attr) is None:
                 g = self.window.get_geometry()
-                self._x = g.x
-                self._y = g.y
-                self._width = g.width
+                self.x = g.x
+                self.y = g.y
+                self.width = g.width
                 self.height = g.height
                 # note that _float_info x and y are
                 # really offsets, relative to screen x,y
@@ -178,7 +182,15 @@ class _Window(command.CommandObject):
         return get_attr
 
     def _geometry_setter(attr):
-        return lambda self, value: setattr(self, "_" + attr, value)
+        def f(self, value):
+            if not isinstance(value, int) and attr != "_float_info":
+                frame = inspect.currentframe()
+                stack_trace = traceback.format_stack(frame)
+                self.qtile.log.error("!!!! setting %s to a non-int %s; please report this!", attr, value)
+                self.qtile.log.error(''.join(stack_trace[:-1]))
+                value = int(value)
+            setattr(self, "_" + attr, value)
+        return f
 
     x = property(fset=_geometry_setter("x"), fget=_geometry_getter("x"))
     y = property(fset=_geometry_setter("y"), fget=_geometry_getter("y"))
@@ -198,7 +210,7 @@ class _Window(command.CommandObject):
     def updateName(self):
         try:
             self.name = self.window.get_name()
-        except (xcb.xproto.BadWindow, xcb.xproto.BadAccess):
+        except (xcffib.xproto.WindowError, xcffib.xproto.AccessError):
             return
         hook.fire("window_name_change")
 
@@ -210,7 +222,7 @@ class _Window(command.CommandObject):
         try:
             h = self.window.get_wm_hints()
             normh = self.window.get_wm_normal_hints()
-        except (xcb.xproto.BadWindow, xcb.xproto.BadAccess):
+        except (xcffib.xproto.WindowError, xcffib.xproto.AccessError):
             return
 
         # FIXME
@@ -239,14 +251,14 @@ class _Window(command.CommandObject):
             if not normh['base_width'] and \
                     normh['min_width'] and \
                     normh['width_inc']:
-                # seems xcb does ignore base width :(
+                # seems xcffib does ignore base width :(
                 normh['base_width'] = (
                     normh['min_width'] % normh['width_inc']
                 )
             if not normh['base_height'] and \
                     normh['min_height'] and \
                     normh['height_inc']:
-                # seems xcb does ignore base height :(
+                # seems xcffib does ignore base height :(
                 normh['base_height'] = (
                     normh['min_height'] % normh['height_inc']
                 )
@@ -266,18 +278,28 @@ class _Window(command.CommandObject):
         return
 
     def updateState(self):
-        if not self.qtile.config.auto_fullscreen:
-            return
+        triggered = ['urgent']
+
+        if self.qtile.config.auto_fullscreen:
+            triggered.append('fullscreen')
+
         state = self.window.get_net_wm_state()
-        self.qtile.log.debug('_NET_WM_STATE: %s' % state)
-        if state == 'fullscreen':
-            self.fullscreen = True
-        else:
-            self.fullscreen = False
+
+        if state:
+            self.qtile.log.debug('_NET_WM_STATE: %s' % ','.join(state))
+            for s in triggered:
+                setattr(self, s, (s in state))
 
     @property
     def urgent(self):
-        return self.hints['urgent']
+        return self.hints['urgent'] or self._demands_attention
+
+    @urgent.setter
+    def urgent(self, val):
+        self._demands_attention = val
+        # TODO unset window hint as well?
+        if not val:
+            self.hints['urgent'] = False
 
     def info(self):
         if self.group:
@@ -317,58 +339,44 @@ class _Window(command.CommandObject):
 
     def getOpacity(self):
         opacity = self.window.get_property(
-            "_NET_WM_WINDOW_OPACITY", unpack="I"
+            "_NET_WM_WINDOW_OPACITY", unpack=int
         )
         if not opacity:
             return 1.0
         else:
             value = opacity[0]
             # 2 decimal places
-            as_float = round((float(value) / 0xffffffff), 2)
+            as_float = round(value / 0xffffffff, 2)
             return as_float
 
     opacity = property(getOpacity, setOpacity)
 
     def kill(self):
         if "WM_DELETE_WINDOW" in self.window.get_wm_protocols():
-            # e = event.ClientMessage(
-            #        window = self.window,
-            #        client_type = self.qtile.display.intern_atom(
-            #             "WM_PROTOCOLS"),
-            #        data = [
-            #            # Use 32-bit format:
-            #            32,
-            #            # Must be exactly 20 bytes long:
-            #            [
-            #                self.qtile.display.intern_atom(
-            #                        "WM_DELETE_WINDOW"),
-            #                X.CurrentTime,
-            #                0,
-            #                0,
-            #                0
-            #            ]
-            #        ]
-            # )
-            vals = [
-                33,  # ClientMessageEvent
-                32,  # Format
-                0,
-                self.window.wid,
-                self.qtile.conn.atoms["WM_PROTOCOLS"],
+            data = [
                 self.qtile.conn.atoms["WM_DELETE_WINDOW"],
-                xcb.xproto.Time.CurrentTime,
-                0,
+                xcffib.xproto.Time.CurrentTime,
                 0,
                 0,
+                0
             ]
-            e = struct.pack('BBHII5I', *vals)
+
+            u = xcffib.xproto.ClientMessageData.synthetic(data, "I" * 5)
+
+            e = xcffib.xproto.ClientMessageEvent.synthetic(
+                format=32,
+                window=self.window.wid,
+                type=self.qtile.conn.atoms["WM_PROTOCOLS"],
+                data=u
+            )
+
             self.window.send_event(e)
         else:
             self.window.kill_client()
 
     def hide(self):
         # We don't want to get the UnmapNotify for this unmap
-        with self.disableMask(xcb.xproto.EventMask.StructureNotify):
+        with self.disableMask(xcffib.xproto.EventMask.StructureNotify):
             self.window.unmap()
         self.hidden = True
 
@@ -460,15 +468,24 @@ class _Window(command.CommandObject):
         window = self.window.wid
         above_sibling = False
         override_redirect = False
-        event_type = 22  # ConfigureNotify
 
-        event = struct.pack('bx2xIIIhhHHHB5x', event_type,
-                            window, window, above_sibling,
-                            x, y, width, height, self.borderwidth,
-                            override_redirect)
+        event = xcffib.xproto.ConfigureNotifyEvent.synthetic(
+            event=window,
+            window=window,
+            above_sibling=above_sibling,
+            x=x,
+            y=y,
+            width=width,
+            height=height,
+            border_width=self.borderwidth,
+            override_redirect=override_redirect
+        )
 
         self.window.send_event(event, mask=EventMask.StructureNotify)
 
+    def can_steal_focus(self):
+        return self.window.get_wm_type() != 'notification'
+
     def focus(self, warp):
 
         # Workaround for misbehaving java applications (actually it might be
@@ -491,19 +508,22 @@ class _Window(command.CommandObject):
             # Never send TAKE_FOCUS on java *dialogs*
             if not is_java_dialog and \
                     "WM_TAKE_FOCUS" in self.window.get_wm_protocols():
-                vals = [
-                    33,
-                    32,
-                    0,
-                    self.window.wid,
-                    self.qtile.conn.atoms["WM_PROTOCOLS"],
+                data = [
                     self.qtile.conn.atoms["WM_TAKE_FOCUS"],
-                    xcb.xproto.Time.CurrentTime,
-                    0,
+                    xcffib.xproto.Time.CurrentTime,
                     0,
                     0,
+                    0
                 ]
-                e = struct.pack('BBHII5I', *vals)
+
+                u = xcffib.xproto.ClientMessageData.synthetic(data, "I" * 5)
+                e = xcffib.xproto.ClientMessageEvent.synthetic(
+                    format=32,
+                    window=self.window.wid,
+                    type=self.qtile.conn.atoms["WM_PROTOCOLS"],
+                    data=u
+                )
+
                 self.window.send_event(e)
 
             # Never send FocusIn to java windows
@@ -514,6 +534,7 @@ class _Window(command.CommandObject):
                     self.window.warp_pointer(self.width // 2, self.height // 2)
             except AttributeError:
                 pass
+        self.urgent = False
         self.qtile.root.set_property("_NET_ACTIVE_WINDOW", self.window.wid)
         hook.fire("client_focus", self)
 
@@ -638,7 +659,7 @@ class Static(_Window):
         self.update_strut()
 
     def handle_ConfigureRequest(self, e):
-        cw = xcb.xproto.ConfigWindow
+        cw = xcffib.xproto.ConfigWindow
         if self.conf_x is None and e.value_mask & cw.X:
             self.x = e.x
         if self.conf_y is None and e.value_mask & cw.Y:
@@ -661,11 +682,11 @@ class Static(_Window):
     def update_strut(self):
         strut = self.window.get_property(
             "_NET_WM_STRUT_PARTIAL",
-            unpack="I" * 12
+            unpack=int
         )
         strut = strut or self.window.get_property(
             "_NET_WM_STRUT",
-            unpack="I" * 4
+            unpack=int
         )
         strut = strut or (0, 0, 0, 0)
         self.qtile.update_gaps(strut, self.strut)
@@ -687,16 +708,17 @@ class Window(_Window):
         EventMask.FocusChange
     # Set when this object is being retired.
     defunct = False
-    _group = None
 
     def __init__(self, window, qtile):
         _Window.__init__(self, window, qtile)
+        self._group = None
         self.updateName()
         # add to group by position according to _NET_WM_DESKTOP property
         index = window.get_wm_desktop()
-        if index and index < len(qtile.groups):
+        if index is not None and index < len(qtile.groups):
             group = qtile.groups[index]
             group.add(self)
+            self._group = group
             if group != qtile.currentScreen.group:
                 self.hide()
 
@@ -711,10 +733,13 @@ class Window(_Window):
     @group.setter
     def group(self, group):
         if group:
-            self.window.set_property(
-                "_NET_WM_DESKTOP",
-                self.qtile.groups.index(group)
-            )
+            try:
+                self.window.set_property(
+                    "_NET_WM_DESKTOP",
+                    self.qtile.groups.index(group)
+                )
+            except xcffib.xproto.WindowError:
+                self.qtile.log.exception("whoops, got error setting _NET_WM_DESKTOP, too early?")
         self._group = group
 
     @property
@@ -814,7 +839,6 @@ class Window(_Window):
             self.group.remove(self)
             screen.group.add(self)
             self.qtile.toScreen(screen.index)
-            # TODO - need to kick boxes to update
 
         self._reconfigure_floating()
 
@@ -879,32 +903,35 @@ class Window(_Window):
                     self.group is not None and \
                     self.group.screen is not None and \
                     screen != self.group.screen:
-                self.x = self.group.screen.x
-                self.y = self.group.screen.y
+                x = self.group.screen.x
+                y = self.group.screen.y
+            else:
+                x = self.x
+                y = self.y
 
             if self.width < self.hints.get('min_width', 0):
-                self.width = self.hints['min_width']
+                width = self.hints['min_width']
+            else:
+                width = self.width
 
             if self.height < self.hints.get('min_height', 0):
-                self.height = self.hints['min_height']
+                height = self.hints['min_height']
+            else:
+                height = self.height
 
-            width = self.width
             if self.hints.get('width_inc', 0):
                 width = (width -
-                    ((width - self.hints['base_width']) %
-                    self.hints['width_inc']))
+                         ((width - self.hints['base_width']) %
+                          self.hints['width_inc']))
 
-            height = self.height
             if self.hints.get('height_inc', 0):
                 height = (height -
-                    ((height - self.hints['base_height'])
-                    % self.hints['height_inc']))
+                          ((height - self.hints['base_height'])
+                           % self.hints['height_inc']))
 
             self.place(
-                self.x,
-                self.y,
-                width,
-                height,
+                x, y,
+                width, height,
                 self.borderwidth,
                 self.bordercolor,
                 above=True,
@@ -964,12 +991,12 @@ class Window(_Window):
             Match window against given attributes.
 
             - wname matches against the window name or title, that is,
-            either `_NET_WM_VISIBLE_NAME`, `_NET_WM_NAME`, `WM_NAME`.
+            either ``_NET_WM_VISIBLE_NAME``, ``_NET_WM_NAME``, ``WM_NAME``.
 
             - wmclass matches against any of the two values in the
-            `WM_CLASS` property
+            ``WM_CLASS`` property
 
-            - role matches against the `WM_WINDOW_ROLE` property
+            - role matches against the ``WM_WINDOW_ROLE`` property
         """
         if not (wname or wmclass or role):
             raise TypeError(
@@ -986,7 +1013,7 @@ class Window(_Window):
             clirole = self.window.get_wm_window_role()
             if role and clirole and role == clirole:
                 return True
-        except (xcb.xproto.BadWindow, xcb.xproto.BadAccess):
+        except (xcffib.xproto.WindowError, xcffib.xproto.AccessError):
             return False
 
         return False
@@ -999,7 +1026,7 @@ class Window(_Window):
         if self.group.screen and \
                 self.qtile.currentScreen != self.group.screen and \
                 self.qtile.config.follow_mouse_focus:
-            self.qtile.toScreen(self.group.screen.index)
+            self.qtile.toScreen(self.group.screen.index, False)
         return True
 
     def handle_ConfigureRequest(self, e):
@@ -1008,24 +1035,19 @@ class Window(_Window):
             return
         if getattr(self, 'floating', False):
             # only obey resize for floating windows
-            cw = xcb.xproto.ConfigWindow
-            if e.value_mask & cw.Width:
-                self.width = e.width
-            if e.value_mask & cw.Height:
-                self.height = e.height
-            if e.value_mask & cw.X:
-                self.x = e.x
-            if e.value_mask & cw.Y:
-                self.y = e.y
+            cw = xcffib.xproto.ConfigWindow
+            width = e.width if e.value_mask & cw.Width else self.width
+            height = e.height if e.value_mask & cw.Height else self.height
+            x = e.x if e.value_mask & cw.X else self.x
+            y = e.y if e.value_mask & cw.Y else self.y
+        else:
+            width, height, x, y = self.width, self.height, self.x, self.y
 
         if self.group and self.group.screen:
             self.place(
-                self.x,
-                self.y,
-                self.width,
-                self.height,
-                self.borderwidth,
-                self.bordercolor,
+                x, y,
+                width, height,
+                self.borderwidth, self.bordercolor,
             )
         self.updateState()
         return False
@@ -1035,10 +1057,10 @@ class Window(_Window):
             Set a dict with the icons of the window
         """
 
-        ret = self.window.get_property('_NET_WM_ICON', 'CARDINAL')
-        if not ret:
+        icon = self.window.get_property('_NET_WM_ICON', 'CARDINAL')
+        if not icon:
             return
-        icon = ret.value
+        icon = list(map(ord, icon.value))
 
         icons = {}
         while True:
@@ -1058,7 +1080,7 @@ class Window(_Window):
 
             arr = array.array("B", data)
             for i in range(0, len(arr), 4):
-                mult = (arr[i + 3]) / 255.
+                mult = arr[i + 3] / 255.
                 arr[i + 0] = int(arr[i + 0] * mult)
                 arr[i + 1] = int(arr[i + 1] * mult)
                 arr[i + 2] = int(arr[i + 2] * mult)
@@ -1070,8 +1092,8 @@ class Window(_Window):
     def handle_ClientMessage(self, event):
         atoms = self.qtile.conn.atoms
 
-        opcode = xcb.xproto.ClientMessageData(event, 0, 20).data32[2]
-        data = xcb.xproto.ClientMessageData(event, 12, 20)
+        opcode = event.type
+        data = event.data
         if atoms["_NET_WM_STATE"] == opcode and \
                 self.qtile.config.auto_fullscreen:
             fullscreen_atom = atoms["_NET_WM_STATE_FULLSCREEN"]
@@ -1079,7 +1101,7 @@ class Window(_Window):
             prev_state = self.window.get_property(
                 '_NET_WM_STATE',
                 'ATOM',
-                unpack='I'
+                unpack=int
             )
             if not prev_state:
                 prev_state = []
@@ -1155,7 +1177,7 @@ class Window(_Window):
         if name == "group":
             return (True, None)
         elif name == "layout":
-            return (True, range(len(self.group.layouts)))
+            return (True, list(range(len(self.group.layouts))))
         elif name == "screen":
             return (True, None)
 
