@@ -1,3 +1,27 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (c) 2010 Aldo Cortesi
+# Copyright (c) 2012 Julian Berman
+# Copyright (c) 2014 Björn Lässig
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 keysyms = {
     'XF86ModeLock': 0x1008FF01,
     'XF86MonBrightnessUp': 0x1008FF02,
@@ -157,6 +181,7 @@ keysyms = {
     'XF86TouchpadToggle': 0x1008FFA9,
     'XF86TouchpadOn': 0x1008FFB0,
     'XF86TouchpadOff': 0x1008FFB1,
+    'XF86AudioMicMute': 0x1008FFB2,
     'XF86Switch_VT_1': 0x1008FE01,
     'XF86Switch_VT_2': 0x1008FE02,
     'XF86Switch_VT_3': 0x1008FE03,
